@@ -7,18 +7,17 @@
 #   Toolkit.
 #
 # AUTHOR
-#   Andy Wardley   <abw@kfs.org>
+#   Andy Wardley   <abw@andywardley.com>
 #
 # COPYRIGHT
-#   Copyright (C) 1996-2000 Andy Wardley.  All Rights Reserved.
-#   Copyright (C) 1998-2000 Canon Research Centre Europe Ltd.
+#   Copyright (C) 1996-2002 Andy Wardley.  All Rights Reserved.
+#   Copyright (C) 1998-2002 Canon Research Centre Europe Ltd.
 #
 #   This module is free software; you can redistribute it and/or
 #   modify it under the same terms as Perl itself.
 #
-#------------------------------------------------------------------------
-#
-#   $Id: Template.pm,v 2.25 2001/11/06 15:00:18 abw Exp $
+# REVISION
+#   $Id: Template.pm,v 2.80 2004/10/04 10:24:10 abw Exp $
 #
 #========================================================================
  
@@ -28,20 +27,24 @@ use base qw( Template::Base );
 require 5.005;
 
 use strict;
-use vars qw( $VERSION $AUTOLOAD $ERROR $DEBUG );
+use vars qw( $VERSION $AUTOLOAD $ERROR $DEBUG $BINMODE );
 use Template::Base;
 use Template::Config;
+use Template::Constants;
 use Template::Provider;  
 use Template::Service;
 use File::Basename;
 use File::Path;
-# use Template::Parser;    # autoloaded on demand
 
 ## This is the main version number for the Template Toolkit.
 ## It is extracted by ExtUtils::MakeMaker and inserted in various places.
-$VERSION     = '2.06';
+$VERSION     = '2.14';
 $ERROR       = '';
 $DEBUG       = 0;
+$BINMODE     = 0 unless defined $BINMODE;
+
+# preload all modules if we're running under mod_perl
+Template::Config->preload() if $ENV{ MOD_PERL };
 
 
 #------------------------------------------------------------------------
@@ -53,26 +56,35 @@ $DEBUG       = 0;
 #------------------------------------------------------------------------
 
 sub process {
-    my ($self, $template, $vars, $outstream) = @_;
+    my ($self, $template, $vars, $outstream, @opts) = @_;
     my ($output, $error);
+    my $options = (@opts == 1) && UNIVERSAL::isa($opts[0], 'HASH')
+        ? shift(@opts) : { @opts };
+    
+    $options->{ binmode } = $BINMODE 
+        unless defined $options->{ binmode };
+
+    # we're using this for testing in t/output.t and t/filter.t so 
+    # don't remove it if you don't want tests to fail...
+    $self->DEBUG("set binmode\n") if $DEBUG && $options->{ binmode };
 
     $output = $self->{ SERVICE }->process($template, $vars);
     
     if (defined $output) {
-	$outstream ||= $self->{ OUTPUT };
-	unless (ref $outstream) {
-	    my $outpath = $self->{ OUTPUT_PATH };
-	    $outstream = "$outpath/$outstream" if $outpath;
-	}	
-
-	# send processed template to output stream, checking for error
-	return ($self->error($error))
-	    if ($error = &_output($outstream, $output));
-
-	return 1;
+        $outstream ||= $self->{ OUTPUT };
+        unless (ref $outstream) {
+            my $outpath = $self->{ OUTPUT_PATH };
+            $outstream = "$outpath/$outstream" if $outpath;
+        }	
+        
+        # send processed template to output stream, checking for error
+        return ($self->error($error))
+            if ($error = &_output($outstream, \$output, $options));
+        
+        return 1;
     }
     else {
-	return $self->error($self->{ SERVICE }->error);
+        return $self->error($self->{ SERVICE }->error);
     }
 }
 
@@ -113,10 +125,24 @@ sub context {
 sub _init {
     my ($self, $config) = @_;
 
+    # convert any textual DEBUG args to numerical form
+    my $debug = $config->{ DEBUG };
+    $config->{ DEBUG } = Template::Constants::debug_flags($self, $debug)
+        || return if defined $debug && $debug !~ /^\d+$/;
+    
+    # prepare a namespace handler for any CONSTANTS definition
+    if (my $constants = $config->{ CONSTANTS }) {
+        my $ns  = $config->{ NAMESPACE } ||= { };
+        my $cns = $config->{ CONSTANTS_NAMESPACE } || 'constants';
+        $constants = Template::Config->constants($constants)
+            || return $self->error(Template::Config->error);
+        $ns->{ $cns } = $constants;
+    }
+    
     $self->{ SERVICE } = $config->{ SERVICE }
-	|| Template::Config->service($config)
-	|| return $self->error(Template::Config->error);
-
+        || Template::Config->service($config)
+        || return $self->error(Template::Config->error);
+    
     $self->{ OUTPUT      } = $config->{ OUTPUT } || \*STDOUT;
     $self->{ OUTPUT_PATH } = $config->{ OUTPUT_PATH };
 
@@ -129,49 +155,60 @@ sub _init {
 #------------------------------------------------------------------------
 
 sub _output {
-    my ($where, $text, $binmode) = @_;
+    my ($where, $textref, $options) = @_;
     my $reftype;
     my $error = 0;
     
-    # call a CODE referenc
+    # call a CODE reference
     if (($reftype = ref($where)) eq 'CODE') {
-	&$where($text);
+        &$where($$textref);
     }
     # print to a glob (such as \*STDOUT)
     elsif ($reftype eq 'GLOB') {
-	print $where $text;
+        print $where $$textref;
     }   
     # append output to a SCALAR ref
     elsif ($reftype eq 'SCALAR') {
-	$$where .= $text;
+        $$where .= $$textref;
+    }
+    # push onto ARRAY ref
+    elsif ($reftype eq 'ARRAY') {
+        push @$where, $$textref;
     }
     # call the print() method on an object that implements the method
     # (e.g. IO::Handle, Apache::Request, etc)
     elsif (UNIVERSAL::can($where, 'print')) {
-	$where->print($text);
+        $where->print($$textref);
     }
     # a simple string is taken as a filename
     elsif (! $reftype) {
-	local *FP;
-	# make destination directory if it doesn't exist
-	my $dir = dirname($where);
-	eval { mkpath($dir) unless -d $dir; };
-	if ($@) {
-	    # strip file name and line number from error raised by die()
-	    ($error = $@) =~ s/ at \S+ line \d+\n?$//;
-	}
-	elsif (open(FP, ">$where")) { 
-	    binmode FP if $binmode;
-	    print FP $text;
-	    close FP;
-	}
-	else {
-	    $error  = "$where: $!";
-	}
+        local *FP;
+        # make destination directory if it doesn't exist
+        my $dir = dirname($where);
+        eval { mkpath($dir) unless -d $dir; };
+        if ($@) {
+            # strip file name and line number from error raised by die()
+            ($error = $@) =~ s/ at \S+ line \d+\n?$//;
+        }
+        elsif (open(FP, ">$where")) { 
+            # binmode option can be 1 or a specific layer, e.g. :utf8
+            my $bm = $options->{ binmode };
+            if ($bm && +$bm == 1) { 
+                binmode FP;
+            }
+            elsif ($bm){ 
+                binmode FP, $bm;
+            }
+            print FP $$textref;
+            close FP;
+        }
+        else {
+            $error  = "$where: $!";
+        }
     }
     # give up, we've done our best
     else {
-	$error = "output_handler() cannot determine target type ($where)\n";
+        $error = "output_handler() cannot determine target type ($where)\n";
     }
 
     return $error;
@@ -249,8 +286,8 @@ class) instantiates a new Template object.  A reference to a hash
 array of configuration items may be passed as a parameter.
 
     my $tt = Template->new({
-	INCLUDE_PATH => '/usr/local/templates',
-	EVAL_PERL    => 1,
+    	INCLUDE_PATH => '/usr/local/templates',
+	    EVAL_PERL    => 1,
     }) || die $Template::ERROR, "\n";
 
 A reference to a new Template object is returned, or undef on error.
@@ -272,7 +309,7 @@ folded into a hash array by the constructor.
     my $tt = Template->new(INCLUDE_PATH => '/tmp', POST_CHOMP => 1)
 	|| die $Template::ERROR, "\n";
 
-=head2 process($template, \%vars, $output)
+=head2 process($template, \%vars, $output, %options)
 
 The process() method is called to process a template.  The first
 parameter indicates the input template as one of: a filename relative
@@ -322,8 +359,8 @@ Examples:
 
     # reference to output subroutine
     sub myout {
-	my $output = shift;
-	...
+    	my $output = shift;
+	    ...
     }
     $tt->process('welcome.tt2', $vars, \&myout)
         || die $tt->error(), "\n";
@@ -351,13 +388,34 @@ In an Apache/mod_perl handler:
 	return OK;
     }
 
+After the optional third output argument can come an optional
+reference to a hash or a list of (name, value) pairs providing further
+options for the output.  The only option currently supported is
+"binmode" which, when set to any true value will ensure that files
+created (but not any existing file handles passed) will be set to
+binary mode.
+
+    # either: hash reference of options
+    $tt->process($infile, $vars, $outfile, { binmode => 1 })
+        || die $tt->error(), "\n";
+
+    # or: list of name, value pairs
+    $tt->process($infile, $vars, $outfile, binmode => 1)
+        || die $tt->error(), "\n";
+
+Alternately, the binmode argument can specify a particular IO layer such 
+as ":utf8".
+
+    $tt->process($infile, $vars, $outfile, binmode => ':utf8')
+        || die $tt->error(), "\n";
+
 The OUTPUT configuration item can be used to specify a default output 
 location other than \*STDOUT.  The OUTPUT_PATH specifies a directory
 which should be prefixed to all output locations specified as filenames.
 
     my $tt = Template->new({
-	OUTPUT      => sub { ... },       # default
-	OUTPUT_PATH => '/tmp',
+    	OUTPUT      => sub { ... },       # default
+	    OUTPUT_PATH => '/tmp',
 	...
     }) || die Template->error(), "\n";
 
@@ -403,10 +461,10 @@ operator allowing the object reference itself to be printed to return
 the formatted error string.
 
     $tt->process('somefile') || do {
-	my $error = $tt->error();
-	print "error type: ", $error->type(), "\n";
-	print "error info: ", $error->info(), "\n";
-	print $error, "\n";
+    	my $error = $tt->error();
+	    print "error type: ", $error->type(), "\n";
+    	print "error info: ", $error->info(), "\n";
+	    print $error, "\n";
     };
 
 =head2 service()
@@ -538,8 +596,7 @@ Directory into which output files can be written.
 
 =item DEBUG
 
-Flag which, when enabled, causes any access to an undefined variable
-to be raised as an 'undef' error.
+Enable debugging messages.
 
 =back
 
@@ -880,7 +937,7 @@ Ignored and deleted.
 
 =head1 AUTHOR
 
-Andy Wardley E<lt>abw@kfs.orgE<gt>
+Andy Wardley E<lt>abw@andywardley.comE<gt>
 
 L<http://www.andywardley.com/|http://www.andywardley.com/>
 
@@ -889,13 +946,24 @@ L<http://www.andywardley.com/|http://www.andywardley.com/>
 
 =head1 VERSION
 
-Template Toolkit version 2.06, released on 07 November 2001.
+Template Toolkit version 2.14, released on 04 October 2004.
 
 =head1 COPYRIGHT
 
-  Copyright (C) 1996-2001 Andy Wardley.  All Rights Reserved.
-  Copyright (C) 1998-2001 Canon Research Centre Europe Ltd.
+  Copyright (C) 1996-2004 Andy Wardley.  All Rights Reserved.
+  Copyright (C) 1998-2002 Canon Research Centre Europe Ltd.
 
 This module is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
 
+
+
+=cut
+
+# Local Variables:
+# mode: perl
+# perl-indent-level: 4
+# indent-tabs-mode: nil
+# End:
+#
+# vim: expandtab shiftwidth=4:
