@@ -17,9 +17,8 @@
 #   This module is free software; you can redistribute it and/or
 #   modify it under the same terms as Perl itself.
 # 
-#----------------------------------------------------------------------------
-#
-# $Id: Context.pm,v 2.38 2001/11/06 15:00:19 abw Exp $
+# REVISION
+#   $Id: Context.pm,v 2.91 2004/10/04 10:02:29 abw Exp $
 #
 #============================================================================
 
@@ -28,7 +27,7 @@ package Template::Context;
 require 5.004;
 
 use strict;
-use vars qw( $VERSION $DEBUG $AUTOLOAD );
+use vars qw( $VERSION $DEBUG $AUTOLOAD $DEBUG_FORMAT );
 use base qw( Template::Base );
 
 use Template::Base;
@@ -36,7 +35,8 @@ use Template::Config;
 use Template::Constants;
 use Template::Exception;
 
-$VERSION = sprintf("%d.%02d", q$Revision: 2.38 $ =~ /(\d+)\.(\d+)/);
+$VERSION = sprintf("%d.%02d", q$Revision: 2.91 $ =~ /(\d+)\.(\d+)/);
+$DEBUG_FORMAT = "\n## \$file line \$line : [% \$text %] ##\n";
 
 
 #========================================================================
@@ -76,66 +76,94 @@ $VERSION = sprintf("%d.%02d", q$Revision: 2.38 $ =~ /(\d+)\.(\d+)/);
 sub template {
     my ($self, $name) = @_;
     my ($prefix, $blocks, $defblocks, $provider, $template, $error);
-    my $providers;
+    my ($shortname, $blockname, $providers);
+
+    $self->debug("template($name)") if $self->{ DEBUG };
 
     # references to Template::Document (or sub-class) objects objects, or
     # CODE references are assumed to be pre-compiled templates and are
     # returned intact
     return $name
-	if UNIVERSAL::isa($name, 'Template::Document')
-	    || ref($name) eq 'CODE';
+        if UNIVERSAL::isa($name, 'Template::Document')
+            || ref($name) eq 'CODE';
+
+    $shortname = $name;
 
     unless (ref $name) {
-	# we first look in the BLOCKS hash for a BLOCK that may have 
-	# been imported from a template (via PROCESS)
-	return $template
-	    if ($template = $self->{ BLOCKS }->{ $name });
+        
+        $self->debug("looking for block [$name]") if $self->{ DEBUG };
 
-	# then we iterate through the BLKSTACK list to see if any of the
-	# Template::Documents we're visiting define this BLOCK
-	foreach $blocks (@{ $self->{ BLKSTACK } }) {
-	    return $template
-		if $blocks && ($template = $blocks->{ $name });
-	}
-
-	# now it's time to ask the providers, so we look to see if any 
-	# prefix is specified to indicate the desired provider set.
-	$prefix = undef;
-
-	if ($^O eq 'MSWin32') {
-	    $prefix = $1	# let C:/foo through
-		if $name =~ s/^(\w{2,})://o;
-	}
-	else {
-	    $prefix = $1 
-		if $name =~ s/^(\w+)://o;
-	}
-
-	if (defined $prefix) {
-	    $providers = $self->{ PREFIX_MAP }->{ $prefix } 
-		|| return $self->throw(Template::Constants::ERROR_FILE,
-				      "no providers for template prefix '$prefix'");
-#	    print STDERR "prefix identified: $prefix\n";
-	}
+        # we first look in the BLOCKS hash for a BLOCK that may have 
+        # been imported from a template (via PROCESS)
+        return $template
+            if ($template = $self->{ BLOCKS }->{ $name });
+        
+        # then we iterate through the BLKSTACK list to see if any of the
+        # Template::Documents we're visiting define this BLOCK
+        foreach $blocks (@{ $self->{ BLKSTACK } }) {
+            return $template
+                if $blocks && ($template = $blocks->{ $name });
+        }
+        
+        # now it's time to ask the providers, so we look to see if any 
+        # prefix is specified to indicate the desired provider set.
+        if ($^O eq 'MSWin32') {
+            # let C:/foo through
+            $prefix = $1 if $shortname =~ s/^(\w{2,})://o;
+        }
+        else {
+            $prefix = $1 if $shortname =~ s/^(\w+)://;
+        }
+        
+        if (defined $prefix) {
+            $providers = $self->{ PREFIX_MAP }->{ $prefix } 
+            || return $self->throw( Template::Constants::ERROR_FILE,
+                                    "no providers for template prefix '$prefix'");
+        }
     }
     $providers = $self->{ PREFIX_MAP }->{ default }
-	      || $self->{ LOAD_TEMPLATES }
-        unless $providers;
+        || $self->{ LOAD_TEMPLATES }
+            unless $providers;
 
-    # finally we try the regular template providers which will 
+
+    # Finally we try the regular template providers which will 
     # handle references to files, text, etc., as well as templates
-    # reference by name
-    foreach my $provider (@$providers) {
-	($template, $error) = $provider->fetch($name);
-	return $template unless $error;
-#	return $self->error($template)
-	if ($error == Template::Constants::STATUS_ERROR) {
-	    $self->throw($template) if ref $template;
-	    $self->throw(Template::Constants::ERROR_FILE, $template);
-	}
-    }
+    # reference by name.  If
 
-#    return $self->error("$name: not found");
+    $blockname = '';
+    while ($shortname) {
+        $self->debug("asking providers for [$shortname] [$blockname]") 
+            if $self->{ DEBUG };
+
+        foreach my $provider (@$providers) {
+            ($template, $error) = $provider->fetch($shortname, $prefix);
+            if ($error) {
+                if ($error == Template::Constants::STATUS_ERROR) {
+                    # $template contains exception object
+                    if (UNIVERSAL::isa($template, 'Template::Exception')
+                        && $template->type() eq Template::Constants::ERROR_FILE) {
+                        $self->throw($template);
+                    }
+                    else {
+                        $self->throw( Template::Constants::ERROR_FILE, $template );
+                    }
+                }
+                # DECLINE is ok, carry on
+            }
+            elsif (length $blockname) {
+                return $template 
+                    if $template = $template->blocks->{ $blockname };
+            }
+            else {
+                return $template;
+            }
+        }
+        
+        last if ref $shortname || ! $self->{ EXPOSE_BLOCKS };
+        $shortname =~ s{/([^/]+)$}{} || last;
+        $blockname = length $blockname ? "$1/$blockname" : $1;
+    }
+        
     $self->throw(Template::Constants::ERROR_FILE, "$name: not found");
 }
 
@@ -154,17 +182,20 @@ sub template {
 sub plugin {
     my ($self, $name, $args) = @_;
     my ($provider, $plugin, $error);
-
+    
+    $self->debug("plugin($name, ", defined $args ? @$args : '[ ]', ')')
+        if $self->{ DEBUG };
+    
     # request the named plugin from each of the LOAD_PLUGINS providers in turn
     foreach my $provider (@{ $self->{ LOAD_PLUGINS } }) {
-	($plugin, $error) = $provider->fetch($name, $args, $self);
-	return $plugin unless $error;
-	if ($error == Template::Constants::STATUS_ERROR) {
-	    $self->throw($plugin) if ref $plugin;
-	    $self->throw(Template::Constants::ERROR_PLUGIN, $plugin);
-	}
+        ($plugin, $error) = $provider->fetch($name, $args, $self);
+        return $plugin unless $error;
+        if ($error == Template::Constants::STATUS_ERROR) {
+            $self->throw($plugin) if ref $plugin;
+            $self->throw(Template::Constants::ERROR_PLUGIN, $plugin);
+        }
     }
-
+    
     $self->throw(Template::Constants::ERROR_PLUGIN, "$name: plugin not found");
 }
 
@@ -180,44 +211,54 @@ sub plugin {
 sub filter {
     my ($self, $name, $args, $alias) = @_;
     my ($provider, $filter, $error);
-
+    
+    $self->debug("filter($name, ", 
+                 defined $args  ? @$args : '[ ]', 
+                 defined $alias ? $alias : '<no alias>', ')')
+        if $self->{ DEBUG };
+    
     # use any cached version of the filter if no params provided
     return $filter 
-	if ! $args && ! ref $name
-	    && ($filter = $self->{ FILTER_CACHE }->{ $name });
-
+        if ! $args && ! ref $name
+            && ($filter = $self->{ FILTER_CACHE }->{ $name });
+    
     # request the named filter from each of the FILTERS providers in turn
     foreach my $provider (@{ $self->{ LOAD_FILTERS } }) {
-	$filter = $name, last 
-	    if ref $name;
-
-	($filter, $error) = $provider->fetch($name, $args, $self);
-	last unless $error;
-	if ($error == Template::Constants::STATUS_ERROR) {
-	    $self->throw($filter) if ref $filter;
-	    $self->throw(Template::Constants::ERROR_FILTER, $filter);
-	}
-	# return $self->error($filter)
-	#    if $error == &Template::Constants::STATUS_ERROR;
+        ($filter, $error) = $provider->fetch($name, $args, $self);
+        last unless $error;
+        if ($error == Template::Constants::STATUS_ERROR) {
+            $self->throw($filter) if ref $filter;
+            $self->throw(Template::Constants::ERROR_FILTER, $filter);
+        }
+        # return $self->error($filter)
+        #    if $error == &Template::Constants::STATUS_ERROR;
     }
-
+    
     return $self->error("$name: filter not found")
-	unless $filter;
-
+        unless $filter;
+    
+    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+    # commented out by abw on 19 Nov 2001 to fix problem with xmlstyle
+    # plugin which may re-define a filter by calling define_filter()
+    # multiple times.  With the automatic aliasing/caching below, any
+    # new filter definition isn't seen.  Don't think this will cause
+    # any problems as filters explicitly supplied with aliases will
+    # still work as expected.
+    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
     # alias defaults to name if undefined
-    $alias = $name
-	unless defined($alias) or ref($name) or $args;
+    # $alias = $name
+    #     unless defined($alias) or ref($name) or $args;
 
     # cache FILTER if alias is valid
     $self->{ FILTER_CACHE }->{ $alias } = $filter
-	if $alias;
+        if $alias;
 
     return $filter;
 }
 
 
 #------------------------------------------------------------------------
-# view(\%coonfig)
+# view(\%config)
 # 
 # Create a new Template::View bound to this context.
 #------------------------------------------------------------------------
@@ -226,78 +267,123 @@ sub view {
     my $self = shift;
     require Template::View;
     return Template::View->new($self, @_)
-	|| $self->throw(&Template::Constants::ERROR_VIEW, 
-			$Template::View::ERROR);
+        || $self->throw(&Template::Constants::ERROR_VIEW, 
+                        $Template::View::ERROR);
 }
 
 
 #------------------------------------------------------------------------
-# process($template, \%params)    [% PROCESS template   var = val, ... %]
+# process($template, \%params)         [% PROCESS template var=val ... %]
+# process($template, \%params, $local) [% INCLUDE template var=val ... %]
 #
 # Processes the template named or referenced by the first parameter.
 # The optional second parameter may reference a hash array of variable
-# definitions.  These are set before the template is processed by calling
-# update() on the stash.  Note that the context is not localised and 
-# these, and any other variables set in the template will retain their
-# new values after this method returns.
+# definitions.  These are set before the template is processed by
+# calling update() on the stash.  Note that, unless the third parameter
+# is true, the context is not localised and these, and any other
+# variables set in the template will retain their new values after this
+# method returns.  The third parameter is in place so that this method
+# can handle INCLUDE calls: the stash will be localized.
 #
 # Returns the output of processing the template.  Errors are thrown
 # as Template::Exception objects via die().  
 #------------------------------------------------------------------------
 
 sub process {
-    my ($self, $template, $params) = @_;
+    my ($self, $template, $params, $localize) = @_;
     my ($trim, $blocks) = @$self{ qw( TRIM BLOCKS ) };
     my (@compiled, $name, $compiled);
-    my ($stash, $tblocks, $error, $tmpout);
+    my ($stash, $component, $tblocks, $error, $tmpout);
     my $output = '';
-
-#    my ($blocks, $output);
-#    my $name = $template;
-
+    
     $template = [ $template ] unless ref $template eq 'ARRAY';
-
+    
+    $self->debug("process([ ", join(', '), @$template, ' ], ', 
+                 defined $params ? $params : '<no params>', ', ', 
+                 $localize ? '<localized>' : '<unlocalized>', ')')
+        if $self->{ DEBUG };
+    
     # fetch compiled template for each name specified
     foreach $name (@$template) {
-	push(@compiled, $self->template($name));
+        push(@compiled, $self->template($name));
     }
 
-    # update stash with any new parameters passed
-    $self->{ STASH }->update($params);
-    $stash = $self->{ STASH };
-
-    foreach $name (@$template) {
-	$compiled = shift @compiled;
-	my $element = ref $compiled eq 'CODE' 
-	    ? { (name => (ref $name ? '' : $name), modtime => time()) }
-	: $compiled;
-	$stash->set('component', $element);
-	
-	# merge any local blocks defined in the Template::Document into our
-	# local BLOCKS cache
-	@$blocks{ keys %$tblocks } = values %$tblocks
-	    if UNIVERSAL::isa($compiled, 'Template::Document')
-		&& ($tblocks = $compiled->blocks());
-	
-	if (ref $compiled eq 'CODE') {
-	    $tmpout = &$compiled($self);
-	}
-	elsif (ref $compiled) {
-	    $tmpout = $compiled->process($self);
-	}
-	else {
-	    $self->throw('file', 
-			 "invalid template reference: $compiled");
-	}
-	
-	if ($trim) {
-	    for ($tmpout) {
-		s/^\s+//;
-		s/\s+$//;
-	    }
-	}
-	$output .= $tmpout;
+    if ($localize) {
+        # localise the variable stash with any parameters passed
+        $stash = $self->{ STASH } = $self->{ STASH }->clone($params);
+    } else {
+        # update stash with any new parameters passed
+        $self->{ STASH }->update($params);
+        $stash = $self->{ STASH };
     }
+
+    eval {
+        # save current component
+        eval { $component = $stash->get('component') };
+
+        foreach $name (@$template) {
+            $compiled = shift @compiled;
+            my $element = ref $compiled eq 'CODE' 
+                ? { (name => (ref $name ? '' : $name), modtime => time()) }
+                : $compiled;
+
+            if (UNIVERSAL::isa($component, 'Template::Document')) {
+                $element->{ caller } = $component->{ name };
+                $element->{ callers } = $component->{ callers } || [];
+                push(@{$element->{ callers }}, $element->{ caller });
+            }
+
+            $stash->set('component', $element);
+            
+            unless ($localize) {
+                # merge any local blocks defined in the Template::Document
+                # into our local BLOCKS cache
+                @$blocks{ keys %$tblocks } = values %$tblocks
+                    if UNIVERSAL::isa($compiled, 'Template::Document')
+                    && ($tblocks = $compiled->blocks());
+            }
+            
+            if (ref $compiled eq 'CODE') {
+                $tmpout = &$compiled($self);
+            }
+            elsif (ref $compiled) {
+                $tmpout = $compiled->process($self);
+            }
+            else {
+                $self->throw('file', 
+                             "invalid template reference: $compiled");
+            }
+            
+            if ($trim) {
+                for ($tmpout) {
+                    s/^\s+//;
+                    s/\s+$//;
+                }
+            }
+            $output .= $tmpout;
+
+            # pop last item from callers.  
+            # NOTE - this will not be called if template throws an 
+            # error.  The whole issue of caller and callers should be 
+            # revisited to try and avoid putting this info directly into
+            # the component data structure.  Perhaps use a local element
+            # instead?
+
+            pop(@{$element->{ callers }})
+                if (UNIVERSAL::isa($element, 'Template::Document'));
+        }
+        $stash->set('component', $component);
+    };
+    $error = $@;
+    
+    if ($localize) {
+        # ensure stash is delocalised before dying
+        $self->{ STASH } = $self->{ STASH }->declone();
+    }
+    
+    $self->throw(ref $error 
+                 ? $error : (Template::Constants::ERROR_FILE, $error))
+        if $error;
     
     return $output;
 }
@@ -319,105 +405,9 @@ sub process {
 
 sub include {
     my ($self, $template, $params) = @_;
-    my $trim = $self->{ TRIM };
-    my (@compiled, $name, $compiled);
-    my ($stash, $error, $tmpout);
-    my $output = '';
-
-    $template = [ $template ] unless ref $template eq 'ARRAY';
-
-    # fetch compiled template for each name specified
-    foreach $name (@$template) {
-	push(@compiled, $self->template($name));
-    }
-
-    # localise the variable stash with any parameters passed
-    $stash = $self->{ STASH } = $self->{ STASH }->clone($params);
-
-    eval {
-	foreach $name (@$template) {
-	    $compiled = shift @compiled;
-	    my $element = ref $compiled eq 'CODE' 
-		? { (name => (ref $name ? '' : $name), modtime => time()) }
-		: $compiled;
-	    $stash->set('component', $element);
-
-	    if (ref $compiled eq 'CODE') {
-		$tmpout = &$compiled($self);
-	    }
-	    elsif (ref $compiled) {
-		$tmpout = $compiled->process($self);
-	    }
-	    else {
-		$self->throw('file', 
-			     "invalid template reference: $compiled");
-	    }
-
-	    if ($trim) {
-		for ($tmpout) {
-		    s/^\s+//;
-		    s/\s+$//;
-		}
-	    }
-	    $output .= $tmpout;
-	}
-    };
-    $error = $@;
-    # ensure stash is delocalised before dying
-    $self->{ STASH } = $self->{ STASH }->declone();
-    $self->throw(ref $error 
-		 ? $error : (Template::Constants::ERROR_FILE, $error))
-	if $error;
-					
-    return $output;
+    return $self->process($template, $params, 'localize me!');
 }
 
-sub old_include {
-    my ($self, $template, $params) = @_;
-    my ($error, $blocks);
-    my $output = '';
-    my $name = $template;
-
-    # request compiled template from cache 
-    $template = $self->template($template);
-#	|| die Template::Exception->new(&Template::Constants::ERROR_FILE, 
-#			       $self->{ _ERROR } || "$template: not found" );
-
-    # localise the variable stash with any parameters passed
-    $params ||= { };
-    $params->{ component } = ref $template eq 'CODE' 
-	? { ref $name ? () : (name => $name, modtime => time()) }
-        : $template;
-    $self->{ STASH } = $self->{ STASH }->clone($params);
-
-    eval {
-	if (ref $template eq 'CODE') {
-	    $output = &$template($self);
-	}
-	elsif (ref $template) {
-	    $output = $template->process($self);
-	}
-	else {
-	    die "invalid template reference: $template\n";
-	}
-    };
-    $error = $@;
-
-    $self->{ STASH } = $self->{ STASH }->declone();
-
-    die $error if $error;
-
-    if ($self->{ TRIM }) {
-	for ($output) {
-	    s/^\s+//;
-	    s/\s+$//;
-	}
-    }
-
-    return $output;
-}
-
-
 #------------------------------------------------------------------------
 # insert($file)
 #
@@ -426,23 +416,38 @@ sub old_include {
 
 sub insert {
     my ($self, $file) = @_;
-    my ($providers, $text, $error);
+    my ($prefix, $providers, $text, $error);
     my $output = '';
 
     my $files = ref $file eq 'ARRAY' ? $file : [ $file ];
 
+    $self->debug("insert([ ", join(', '), @$files, " ])") 
+        if $self->{ DEBUG };
+
+
     FILE: foreach $file (@$files) {
-	if ($file =~ s/^(\w{2,})://o) {
-	    $providers = $self->{ PREFIX_MAP }->{ $1 } 
-		|| return $self->throw(Template::Constants::ERROR_FILE,
-				      "no providers for file prefix '$1'");
+	my $name = $file;
+
+	if ($^O eq 'MSWin32') {
+	    # let C:/foo through
+	    $prefix = $1 if $name =~ s/^(\w{2,})://o;
 	}
 	else {
-	    $providers = $self->{ LOAD_TEMPLATES };
+	    $prefix = $1 if $name =~ s/^(\w+)://;
+	}
+
+	if (defined $prefix) {
+	    $providers = $self->{ PREFIX_MAP }->{ $prefix } 
+	    || return $self->throw(Template::Constants::ERROR_FILE,
+				   "no providers for file prefix '$prefix'");
+	}
+	else {
+	    $providers = $self->{ PREFIX_MAP }->{ default }
+	    || $self->{ LOAD_TEMPLATES };
 	}
 
 	foreach my $provider (@$providers) {
-	    ($text, $error) = $provider->load($file);
+	    ($text, $error) = $provider->load($name, $prefix);
 	    next FILE unless $error;
 	    if ($error == Template::Constants::STATUS_ERROR) {
 		$self->throw($text) if ref $text;
@@ -491,17 +496,14 @@ sub throw {
 
     # die! die! die!
     if (UNIVERSAL::isa($error, 'Template::Exception')) {
-#	print STDERR "throwing existing exception [@$error]\n";
 	die $error;
     }
     elsif (defined $info) {
-#	print STDERR "throwing new exception [$error] [$info]\n";
-	die Template::Exception->new($error, $info, $output);
+	die (Template::Exception->new($error, $info, $output));
     }
     else {
 	$error ||= '';
-#	print STDERR "throwing an undefined exception [$error]\n";
-	die Template::Exception->new('undef', $error, $output);
+	die (Template::Exception->new('undef', $error, $output));
     }
 
     # not reached
@@ -565,7 +567,7 @@ sub delocalise {
 
 
 #------------------------------------------------------------------------
-# visit($blocks)
+# visit($document, $blocks)
 #
 # Each Template::Document calls the visit() method on the context
 # before processing itself.  It passes a reference to the hash array
@@ -576,7 +578,7 @@ sub delocalise {
 #------------------------------------------------------------------------
 
 sub visit {
-    my ($self, $blocks) = @_;
+    my ($self, $document, $blocks) = @_;
     unshift(@{ $self->{ BLKSTACK } }, $blocks)
 }
 
@@ -667,6 +669,62 @@ sub stash {
 }
 
 
+#------------------------------------------------------------------------
+# define_vmethod($type, $name, \&sub)
+#
+# Passes $type, $name, and &sub on to stash->define_vmethod().
+#------------------------------------------------------------------------
+sub define_vmethod {
+    my $self = shift;
+    $self->stash->define_vmethod(@_);
+}
+
+
+#------------------------------------------------------------------------
+# debugging($command, @args, \%params)
+#
+# Method for controlling the debugging status of the context.  The first
+# argument can be 'on' or 'off' to enable/disable debugging, 'format'
+# to define the format of the debug message, or 'msg' to generate a 
+# debugging message reporting the file, line, message text, etc., 
+# according to the current debug format.
+#------------------------------------------------------------------------
+
+sub debugging {
+    my $self = shift;
+    my $hash = ref $_[-1] eq 'HASH' ? pop : { };
+    my @args = @_;
+
+#    print "*** debug(@args)\n";
+    if (@args) {
+	if ($args[0] =~ /^on|1$/i) {
+	    $self->{ DEBUG_DIRS } = 1;
+	    shift(@args);
+	}
+	elsif ($args[0] =~ /^off|0$/i) {
+	    $self->{ DEBUG_DIRS } = 0;
+	    shift(@args);
+	}
+    }
+
+    if (@args) {
+	if ($args[0] =~ /^msg$/i) {
+            return unless $self->{ DEBUG_DIRS };
+	    my $format = $self->{ DEBUG_FORMAT };
+	    $format = $DEBUG_FORMAT unless defined $format;
+	    $format =~ s/\$(\w+)/$hash->{ $1 }/ge;
+	    return $format;
+	}
+	elsif ($args[0] =~ /^format$/i) {
+	    $self->{ DEBUG_FORMAT } = $args[1];
+	}
+	# else ignore
+    }
+
+    return '';
+}
+
+
 #------------------------------------------------------------------------
 # AUTOLOAD
 #
@@ -735,7 +793,9 @@ sub _init {
     my $providers  = $self->{ LOAD_TEMPLATES };
     my $prefix_map = $self->{ PREFIX_MAP } = $config->{ PREFIX_MAP } || { };
     while (my ($key, $val) = each %$prefix_map) {
-	$prefix_map->{ $key } = [ map { $providers->[$_] } split(/\D+/, $val) ]
+	$prefix_map->{ $key } = [ ref $val ? $val :
+				  map { $providers->[$_] } 
+				  split(/\D+/, $val) ]
 	    unless ref $val eq 'ARRAY';
 #	print(STDERR "prefix $key => $val => [", 
 #	      join(', ', @{ $prefix_map->{ $key } }), "]\n");
@@ -748,7 +808,10 @@ sub _init {
 		    || { };
 
 	# hack to get stash to know about debug mode
-	$predefs->{ _DEBUG } = $config->{ DEBUG } || 0;
+	$predefs->{ _DEBUG } = ( ($config->{ DEBUG } || 0)
+                               & &Template::Constants::DEBUG_UNDEF ) ? 1 : 0
+            unless defined $predefs->{ _DEBUG };
+                                
 	Template::Config->stash($predefs)
 	    || return $self->error($Template::Config::ERROR);
     };
@@ -769,17 +832,29 @@ sub _init {
     # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
     # RECURSION - flag indicating is recursion into templates is supported
     # EVAL_PERL - flag indicating if PERL blocks should be processed
-#    # DELIMITER - used to detect template prefixes
     # TRIM      - flag to remove leading and trailing whitespace from output
     # BLKSTACK  - list of hashes of BLOCKs defined in current template(s)
     # CONFIG    - original configuration hash
+    # EXPOSE_BLOCKS - make blocks visible as pseudo-files
+    # DEBUG_FORMAT  - format for generating template runtime debugging messages
+    # DEBUG         - format for generating template runtime debugging messages
 
     $self->{ RECURSION } = $config->{ RECURSION } || 0;
     $self->{ EVAL_PERL } = $config->{ EVAL_PERL } || 0;
-#    $self->{ DELIMITER } = $config->{ DELIMITER } || ':';
     $self->{ TRIM      } = $config->{ TRIM } || 0;
     $self->{ BLKSTACK  } = [ ];
     $self->{ CONFIG    } = $config;
+    $self->{ EXPOSE_BLOCKS } = defined $config->{ EXPOSE_BLOCKS }
+                                     ? $config->{ EXPOSE_BLOCKS } 
+                                     : 0;
+
+    $self->{ DEBUG_FORMAT  } =  $config->{ DEBUG_FORMAT };
+    $self->{ DEBUG_DIRS    } = ($config->{ DEBUG } || 0) 
+                               & Template::Constants::DEBUG_DIRS;
+    $self->{ DEBUG } = defined $config->{ DEBUG } 
+        ? $config->{ DEBUG } & ( Template::Constants::DEBUG_CONTEXT
+                               | Template::Constants::DEBUG_FLAGS )
+        : $DEBUG;
 
     return $self;
 }
@@ -794,12 +869,26 @@ sub _init {
 
 sub _dump {
     my $self = shift;
-    my $output = "$self\n";
-    foreach my $pname (qw( LOAD_TEMPLATES LOAD_PLUGINS LOAD_FILTERS )) {
-	foreach my $prov (@{ $self->{ $pname } }) {
-	    $output .= $prov->_dump();
-	}
+    my $output = "[Template::Context] {\n";
+    my $format = "    %-16s => %s\n";
+    my $key;
+
+    foreach $key (qw( RECURSION EVAL_PERL TRIM )) {
+	$output .= sprintf($format, $key, $self->{ $key });
     }
+    foreach my $pname (qw( LOAD_TEMPLATES LOAD_PLUGINS LOAD_FILTERS )) {
+	my $provtext = "[\n";
+	foreach my $prov (@{ $self->{ $pname } }) {
+	    $provtext .= $prov->_dump();
+#	    $provtext .= ",\n";
+	}
+	$provtext =~ s/\n/\n        /g;
+	$provtext =~ s/\s+$//;
+	$provtext .= ",\n    ]";
+	$output .= sprintf($format, $pname, $provtext);
+    }
+    $output .= sprintf($format, STASH => $self->{ STASH }->_dump());
+    $output .= '}';
     return $output;
 }
 
@@ -1236,6 +1325,52 @@ version 1.
 
 
 
+=item DEBUG
+
+The DEBUG option can be used to enable various debugging features
+of the Template::Context module.  
+
+    use Template::Constants qw( :debug );
+
+    my $template = Template->new({
+	DEBUG => DEBUG_CONTEXT | DEBUG_DIRS,
+    });
+
+The DEBUG value can include any of the following.  Multiple values
+should be combined using the logical OR operator, '|'.
+
+=over 4
+
+=item DEBUG_CONTEXT
+
+Enables general debugging messages for the
+L<Template::Context|Template::Context> module.
+
+=item DEBUG_DIRS
+
+This option causes the Template Toolkit to generate comments
+indicating the source file, line and original text of each directive
+in the template.  These comments are embedded in the template output
+using the format defined in the DEBUG_FORMAT configuration item, or a
+simple default format if unspecified.
+
+For example, the following template fragment:
+
+    
+    Hello World
+
+would generate this output:
+
+    ## input text line 1 :  ##
+    Hello 
+    ## input text line 2 : World ##
+    World
+
+
+=back
+
+
+
 
 
 =back
@@ -1371,7 +1506,7 @@ throws a 'filter' exception on error.
 =head2 localise(\%vars)
 
 Clones the stash to create a context with localised variables.  Returns a 
-reference to the newly cloned Template::Stash object which is also stored
+reference to the newly cloned stash object which is also stored
 internally.
 
     $stash = $context->localise();
@@ -1410,7 +1545,7 @@ An AUTOLOAD method provides access to context configuration items.
 
 =head1 AUTHOR
 
-Andy Wardley E<lt>abw@kfs.orgE<gt>
+Andy Wardley E<lt>abw@andywardley.comE<gt>
 
 L<http://www.andywardley.com/|http://www.andywardley.com/>
 
@@ -1419,13 +1554,13 @@ L<http://www.andywardley.com/|http://www.andywardley.com/>
 
 =head1 VERSION
 
-2.38, distributed as part of the
-Template Toolkit version 2.06, released on 07 November 2001.
+2.91, distributed as part of the
+Template Toolkit version 2.14, released on 04 October 2004.
 
 =head1 COPYRIGHT
 
-  Copyright (C) 1996-2001 Andy Wardley.  All Rights Reserved.
-  Copyright (C) 1998-2001 Canon Research Centre Europe Ltd.
+  Copyright (C) 1996-2004 Andy Wardley.  All Rights Reserved.
+  Copyright (C) 1998-2002 Canon Research Centre Europe Ltd.
 
 This module is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
@@ -1433,3 +1568,13 @@ modify it under the same terms as Perl itself.
 =head1 SEE ALSO
 
 L<Template|Template>, L<Template::Document|Template::Document>, L<Template::Exception|Template::Exception>, L<Template::Filters|Template::Filters>, L<Template::Plugins|Template::Plugins>, L<Template::Provider|Template::Provider>, L<Template::Service|Template::Service>, L<Template::Stash|Template::Stash>
+
+=cut
+
+# Local Variables:
+# mode: perl
+# perl-indent-level: 4
+# indent-tabs-mode: nil
+# End:
+#
+# vim: expandtab shiftwidth=4:
