@@ -19,7 +19,7 @@
 # 
 #----------------------------------------------------------------------------
 #
-# $Id: Service.pm,v 2.36 2001/11/06 15:00:19 abw Exp $
+# $Id: Service.pm,v 2.77 2004/09/17 07:31:30 abw Exp $
 #
 #============================================================================
 
@@ -33,8 +33,9 @@ use base qw( Template::Base );
 use Template::Base;
 use Template::Config;
 use Template::Exception;
+use Template::Constants;
 
-$VERSION = sprintf("%d.%02d", q$Revision: 2.36 $ =~ /(\d+)\.(\d+)/);
+$VERSION = sprintf("%d.%02d", q$Revision: 2.77 $ =~ /(\d+)\.(\d+)/);
 $DEBUG   = 0 unless defined $DEBUG;
 
 
@@ -59,57 +60,76 @@ sub process {
     my ($name, $output, $procout, $error);
     $output = '';
 
+    $self->debug("process($template, ", 
+                 defined $params ? $params : '<no params>',
+                 ')') if $self->{ DEBUG };
+
     $context->reset()
-	if $self->{ AUTO_RESET };
+        if $self->{ AUTO_RESET };
 
     # pre-request compiled template from context so that we can alias it 
     # in the stash for pre-processed templates to reference
     eval { $template = $context->template($template) };
     return $self->error($@)
-	if $@;
+        if $@;
 
     # localise the variable stash with any parameters passed
     # and set the 'template' variable
     $params ||= { };
     $params->{ template } = $template 
-	unless ref $template eq 'CODE';
+        unless ref $template eq 'CODE';
     $context->localise($params);
 
     SERVICE: {
-	# PRE_PROCESS
-	eval {
-	    foreach $name (@{ $self->{ PRE_PROCESS } }) {
-		$output .= $context->process($name);
-	    }
-	};
-	last SERVICE if ($error = $@);
+        # PRE_PROCESS
+        eval {
+            foreach $name (@{ $self->{ PRE_PROCESS } }) {
+                $self->debug("PRE_PROCESS: $name") if $self->{ DEBUG };
+                $output .= $context->process($name);
+            }
+        };
+        last SERVICE if ($error = $@);
 
-	# PROCESS
-	eval {
-	    foreach $name (@{ $self->{ PROCESS } || [ $template ] }) {
-		$procout .= $context->process($name);
-	    }
-	};
-	if ($error = $@) {
-	    last SERVICE
-		unless defined ($procout = $self->_recover(\$error));
-	}
-	$output .= $procout if defined $procout;
-
-	# POST_PROCESS
-	eval {
-	    foreach $name (@{ $self->{ POST_PROCESS } }) {
-		$output .= $context->process($name);
-	    }
-	};
-	last SERVICE if ($error = $@);
+        # PROCESS
+        eval {
+            foreach $name (@{ $self->{ PROCESS } || [ $template ] }) {
+                $self->debug("PROCESS: $name") if $self->{ DEBUG };
+                $procout .= $context->process($name);
+            }
+        };
+        if ($error = $@) {
+            last SERVICE
+                unless defined ($procout = $self->_recover(\$error));
+        }
+        
+        if (defined $procout) {
+            # WRAPPER
+            eval {
+                foreach $name (reverse @{ $self->{ WRAPPER } }) {
+                    $self->debug("WRAPPER: $name") if $self->{ DEBUG };
+                    $procout = $context->process($name, { content => $procout });
+                }
+            };
+            last SERVICE if ($error = $@);
+            $output .= $procout;
+        }
+        
+        # POST_PROCESS
+        eval {
+            foreach $name (@{ $self->{ POST_PROCESS } }) {
+                $self->debug("POST_PROCESS: $name") if $self->{ DEBUG };
+                $output .= $context->process($name);
+            }
+        };
+        last SERVICE if ($error = $@);
     }
 
     $context->delocalise();
+    delete $params->{ template };
 
     if ($error) {
-#	$error = $error->as_string if ref $error;
-	return $self->error($error);
+    #	$error = $error->as_string if ref $error;
+        return $self->error($error);
     }
 
     return $output;
@@ -139,25 +159,27 @@ sub _init {
 
     # coerce PRE_PROCESS, PROCESS and POST_PROCESS to arrays if necessary, 
     # by splitting on non-word characters
-    foreach $item (qw( PRE_PROCESS PROCESS POST_PROCESS )) {
-	$data = $config->{ $item };
-	next unless defined $data;
-	$data = [ split($delim, $data || '') ]
-	    unless ref $data eq 'ARRAY';
+    foreach $item (qw( PRE_PROCESS PROCESS POST_PROCESS WRAPPER )) {
+        $data = $config->{ $item };
+        $self->{ $item } = [ ], next unless (defined $data);
+        $data = [ split($delim, $data || '') ]
+            unless ref $data eq 'ARRAY';
         $self->{ $item } = $data;
     }
     # unset PROCESS option unless explicitly specified in config
     $self->{ PROCESS } = undef
-	unless exists $config->{ PROCESS };
+        unless defined $config->{ PROCESS };
     
     $self->{ ERROR      } = $config->{ ERROR } || $config->{ ERRORS };
     $self->{ AUTO_RESET } = defined $config->{ AUTO_RESET }
-			  ? $config->{ AUTO_RESET } : 1;
-
+                            ? $config->{ AUTO_RESET } : 1;
+    $self->{ DEBUG      } = ( $config->{ DEBUG } || 0 )
+                            & Template::Constants::DEBUG_SERVICE;
+    
     $context = $self->{ CONTEXT } = $config->{ CONTEXT }
         || Template::Config->context($config)
-	|| return $self->error(Template::Config->error);
-
+        || return $self->error(Template::Config->error);
+    
     return $self;
 }
 
@@ -187,39 +209,42 @@ sub _recover {
     # a 'stop' exception is thrown by [% STOP %] - we return the output
     # buffer stored in the exception object
     return $$error->text()
-	if $$error->type() eq 'stop';
+        if $$error->type() eq 'stop';
 
     my $handlers = $self->{ ERROR }
         || return undef;					## RETURN
 
     if (ref $handlers eq 'HASH') {
-	if ($hkey = $$error->select_handler(keys %$handlers)) {
-	    $handler = $handlers->{ $hkey };
-	}
-	elsif ($handler = $handlers->{ default }) {
-	    # use default handler
-	}
-	else {
-	    return undef;					## RETURN
-	}
+        if ($hkey = $$error->select_handler(keys %$handlers)) {
+            $handler = $handlers->{ $hkey };
+            $self->debug("using error handler for $hkey") if $self->{ DEBUG };
+        }
+        elsif ($handler = $handlers->{ default }) {
+            # use default handler
+            $self->debug("using default error handler") if $self->{ DEBUG };
+        }
+        else {
+            return undef;					## RETURN
+        }
     }
     else {
-	$handler = $handlers;
+        $handler = $handlers;
+        $self->debug("using default error handler") if $self->{ DEBUG };
     }
-
+    
     eval { $handler = $context->template($handler) };
     if ($@) {
-	$$error = $@;
-	return undef;						## RETURN
+        $$error = $@;
+        return undef;						## RETURN
     };
-
+    
     $context->stash->set('error', $$error);
     eval {
-	$output .= $context->process($handler);
+        $output .= $context->process($handler);
     };
     if ($@) {
-	$$error = $@;
-	return undef;						## RETURN
+        $$error = $@;
+        return undef;						## RETURN
     }
 
     return $output;
@@ -633,7 +658,7 @@ or by calling the throw() method on the current Template::Context object,
 
 or from Perl code by calling die() with a Template::Exception object,
 
-    die Template::Exception->new('user.denied', 'Invalid User ID');
+    die (Template::Exception->new('user.denied', 'Invalid User ID'));
 
 or by simply calling die() with an error string.  This is
 automagically caught and converted to an  exception of 'undef'
@@ -668,6 +693,21 @@ is enabled (default), or if the Template::Context reset() method is called.
 
 
 
+=item DEBUG
+
+The DEBUG option can be used to enable debugging messages from the
+Template::Service module by setting it to include the DEBUG_SERVICE
+value.
+
+    use Template::Constants qw( :debug );
+
+    my $template = Template->new({
+	DEBUG => DEBUG_SERVICE,
+    });
+
+
+
+
 =back
 
 =head2 process($input, \%replace)
@@ -700,7 +740,7 @@ Returns the most recent error message.
 
 =head1 AUTHOR
 
-Andy Wardley E<lt>abw@kfs.orgE<gt>
+Andy Wardley E<lt>abw@andywardley.comE<gt>
 
 L<http://www.andywardley.com/|http://www.andywardley.com/>
 
@@ -709,13 +749,13 @@ L<http://www.andywardley.com/|http://www.andywardley.com/>
 
 =head1 VERSION
 
-2.34, distributed as part of the
-Template Toolkit version 2.06, released on 07 November 2001.
+2.81, distributed as part of the
+Template Toolkit version 2.14, released on 04 October 2004.
 
 =head1 COPYRIGHT
 
-  Copyright (C) 1996-2001 Andy Wardley.  All Rights Reserved.
-  Copyright (C) 1998-2001 Canon Research Centre Europe Ltd.
+  Copyright (C) 1996-2004 Andy Wardley.  All Rights Reserved.
+  Copyright (C) 1998-2002 Canon Research Centre Europe Ltd.
 
 This module is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
@@ -723,3 +763,13 @@ modify it under the same terms as Perl itself.
 =head1 SEE ALSO
 
 L<Template|Template>, L<Template::Context|Template::Context>
+
+=cut
+
+# Local Variables:
+# mode: perl
+# perl-indent-level: 4
+# indent-tabs-mode: nil
+# End:
+#
+# vim: expandtab shiftwidth=4:
